/*
 * Linux Event Logging for the Enterprise
 * Copyright (C) International Business Machines Corp., 2002
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Please send e-mail to nguyhien@users.sourceforge.net if you have
 *  questions or comments.
 *
 *  Project Website:  http://evlog.sourceforge.net/
 *
 */
#include <sys/types.h>
#include <stdlib.h>
#include <stdio.h>
#include <dirent.h>
#include <errno.h>
#include <evl_template.h>

static const char *progName;
static const char *facility;

static void report(template_t *t);

static void usage()
{
	fprintf(stderr, "usage: %s -f facility\n", progName);
	exit(1);
}

main(int argc, char **argv)
{
	struct dirent *dirent;
	DIR *dir;

	progName = argv[0];
	if (argc != 3 || strcmp(argv[1], "-f") != 0) {
		usage();
	}
	facility = argv[2];

	dir = opendir(".");
	if (!dir) {
		perror(".");
		exit(1);
	}
	while ((dirent = readdir(dir)) != NULL) {
		template_t *t;
		const char *fname = dirent->d_name;
		if (!_evlStartsWith(fname, "0x")
		    || !_evlEndsWith(fname, ".to")) {
			continue;
		}
		t = _evlReadTemplate(fname);
		if (!t) {
			/* _evlReadTemplate has reported the problem. */
			continue;
		}
		report(t);
	}
	(void) closedir(dir);
}

static evlattribute_t *
getAtt(template_t *t, const char *attName)
{
	evlattribute_t *att;
	int status = evltemplate_getatt(t, attName, &att);
	if (status != 0) {
		fprintf(stderr,
			"%s: Can't find attribute %s for event type %#x\n",
			progName, attName, t->tm_header.u.u_evl.evl_event_type);
		return NULL;
	}
	return att;
}

static void
report(template_t *t)
{
	evlattribute_t *att;

	att = getAtt(t, "eventName");
	if (!att) {
		return;
	}
	printf("%s_%s {\n", facility, evl_getStringAttVal(att));
	printf("\tfilter = 'facility=\"%s\" && event_type=%#x'\n",
		facility, t->tm_header.u.u_evl.evl_event_type);

	att = getAtt(t, "threshold");
	if (att) {
		printf("\tthreshold = %d\n", (int) evl_getLongAttVal(att));
	}

	att = getAtt(t, "interval");
	if (att) {
		printf("\tinterval = %d\n", (int) evl_getLongAttVal(att));
	}
	printf("}\n");
}
