#include "ela.h"

facility "e1000";
event_type  DPFX_FMT "No usable DMA configuration, aborting\n";
const {
	string eventName = "no_dma_cfg";
	string file = "drivers/net/e1000/e1000_main.c";
	string function = "e1000_probe";
	ElaClass class = HARDWARE;
	ElaType type = PERM;
	ElaStringList probableCauses = {
		"Unable to enable DMA for either 32 or 64 bits"
	};
	ElaStringList actions = {
		"Verify driver software level",
		"Execute diagnostics",
		"Replace adapter"
	};
	string runcmd="/usr/share/evlog/ela_hw_alert";
	int threshold = 1;
	int interval = -1;
}

attributes {
	string msg;
	int argsz;
	DPFX_ATTS
}
format string "%msg:printk%"
END

facility "e1000";
event_type NPFX_FMT "The EEPROM Checksum Is Not Valid\n";
const {
	string eventName = "invalid_eeprom_cksum";
	string file = "drivers/net/e1000/e1000_main.c";
	string function = "e1000_probe";
	ElaClass class = HARDWARE;
	ElaType type = PERM;
	ElaStringList probableCauses = {
		"Unable to access PCI data area for checksum value.  However, device ID is valid."
	};
	ElaStringList actions = {
		"Verify driver software level",
		"Execute diagnostics",
		"Replace adapter"
	};
	string runcmd="/usr/share/evlog/ela_hw_alert";
	int threshold = 1;
	int interval = -1;
}

attributes {
	string msg;
	int argsz;
	NPFX_ATTS
}
format string "%msg:printk%"
END

facility "e1000";
event_type NPFX_FMT "Unknown MAC Type\n";
const {
	string eventName = "unk_mac_type";
	string file = "drivers/net/e1000/e1000_main.c";
	string function = "e1000_sw_init";
	ElaClass class = HARDWARE;
	ElaType type = PERM;
	ElaStringList probableCauses = {
		"The MAC type is not ethernet as expected."
	};
	ElaStringList actions = {
		"Verify driver software level",
		"Execute diagnostics",
		"Replace adapter"
	};
	string runcmd="/usr/share/evlog/ela_hw_alert";
	int threshold = 1;
	int interval = -1;
}

attributes {
	string msg;
	int argsz;
	NPFX_ATTS
}
format string "%msg:printk%"
END

facility "e1000";
event_type NPFX_FMT "Invalid MTU setting\n";
const {
	string eventName = "invalid_mtu";
	string file = "drivers/net/e1000/e1000_main.c";
	string function = "e1000_change_mtu";
	ElaClass class = HARDWARE;
	ElaType type = PERM;
/*
 * Given the below Actions, shouldn't this be
 * Class = SOFTWARE, Type = CONFIG?
 */
	ElaStringList probableCauses = {
		"Maximum transmission unit (packet size) is incorrectly set.  "
			"This value must be between 64 and 16128."
	};
	ElaStringList actions = {
		"Use ifconfig (or some other network configuration tool) to set "
			"the MTU value within range"
	};
	string runcmd="/usr/share/evlog/ela_hw_alert";
	int threshold = 1;
	int interval = -1;
}

attributes {
	string msg;
	int argsz;
	NPFX_ATTS
}
format string "%msg:printk%"
END

facility "e1000";
event_type NPFX_FMT "Warning: no configuration for board #%i\n";
const {
	string eventName = "no_cfg";
	string file = "drivers/net/e1000/e1000_param.c";
	string function = "e1000_check_options";
	ElaClass class = SOFTWARE;
	ElaType type = CONFIG;
	ElaStringList probableCauses = {
		"E1000_MAX_NIC is too low in e1000_param.c "
			"or the module parameter (same name E1000_MAX_NIC) "
			"was set below the number of actual network interface cards "
			"in the system."
	};
	ElaStringList actions = {
		"Change E1000_MAX_NIC in e1000_param.c and rebuild the driver, "
			"or change the startup module parameter."
	};
	string runcmd="/usr/share/evlog/ela_hw_alert";
	int threshold = 1;
	int interval = -1;
}

attributes {
	string msg;
	int argsz;
	NPFX_ATTS
	int board;
}
format string "%msg:printk%"
END
