#ifndef lint
static char const 
yyrcsid[] = "$FreeBSD: src/usr.bin/yacc/skeleton.c,v 1.28 2000/01/17 02:04:06 bde Exp $";
#endif
#include <stdlib.h>
#define YYBYACC 1
#define YYMAJOR 1
#define YYMINOR 9
#define YYLEX yylex()
#define YYEMPTY -1
#define yyclearin (yychar=(YYEMPTY))
#define yyerrok (yyerrflag=0)
#define YYRECOVERING() (yyerrflag!=0)
static int yygrowstack();
#define yyparse ttparse
#define yylex ttlex
#define yyerror tterror
#define yychar ttchar
#define yyval ttval
#define yylval ttlval
#define yydebug ttdebug
#define yynerrs ttnerrs
#define yyerrflag tterrflag
#define yyss ttss
#define yyssp ttssp
#define yyvs ttvs
#define yyvsp ttvsp
#define yylhs ttlhs
#define yylen ttlen
#define yydefred ttdefred
#define yydgoto ttdgoto
#define yysindex ttsindex
#define yyrindex ttrindex
#define yygindex ttgindex
#define yytable tttable
#define yycheck ttcheck
#define yyname ttname
#define yyrule ttrule
#define yysslim ttsslim
#define yystacksize ttstacksize
#define YYPREFIX "tt"
/*
 * Linux Event Logging for the Enterprise
 * Copyright (c) International Business Machines Corp., 2001
 *
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *  Please send e-mail to lkessler@users.sourceforge.net if you have
 *  questions or comments.
 *
 *  Project Website:  http://evlog.sourceforge.net/
 *
 */

/*
Template grammar

Note:
This grammar has 1 shift-reduce conflict, which arises from the ambiguity
in the following valid statement:
	string s = "Help!" "%-20s";
This statement is correctly (but perhaps counter-intuitively) parsed as
	string s = "Help!%-20s";
To suppress string concatenation in this case, do
	string s = {"Help!"} "%-20s";
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <assert.h>
#include <wchar.h>
#include <limits.h>

#include "evl_template.h"

posix_log_facility_t evltc_default_facility = EVL_INVALID_FACILITY;

static char *wcsToMbs(wchar_t *ws);
static int makeIntType(evl_list_t *modifiers);
static tmpl_parser_context_t parserContext;

static template_t *template = 0;	/* The template we build */
static int attrClass = 0;
static int earlyTmplFlags = 0x0; /* Flags accumulated before template created */
typedef union {
	int		ival;
	unsigned int	uival;
	unsigned long	ulval;
	long		lval;
	double		dval;
	char		*sval;
	char		**sarrval;
	int		cval;	/* character */
	wchar_t		wcval;
	wchar_t		*wsval;
	tmpl_data_type_t	*tyval;		/* attribute's data type */
	tmpl_dimension_t	*dimval;	/* attribute's dimension */
	tmpl_type_and_value_t	*valval;	/* initializer */
	tmpl_delimiter_t	*delimval;	/* attribute's delimiter */
	evl_list_t	*listval;
	posix_log_facility_t facval;
} YYSTYPE;
#define YYERRCODE 256
#define TK_DELIMITER 257
#define TK_TYPENAME 258
#define TK_NAME 259
#define TK_INTEGER 260
#define TK_DOUBLE 261
#define TK_STRING 262
#define TK_CHAR 263
#define TK_WSTRING 264
#define TK_WCHAR 265
#define TK_FORMAT 266
#define TK_FORMAT_STRING 267
#define TK_FORMAT_WSTRING 268
#define TK_ALIGNED 269
#define TK_ATTRIBUTES 270
#define TK_CONST 271
#define TK_DESCRIPTION 272
#define TK_DEFAULT 273
#define TK_EVENT_TYPE 274
#define TK_FACILITY 275
#define TK_IMPORT 276
#define TK_STRUCT 277
#define TK_TYPEDEF 278
#define TK_TYCHAR 279
#define TK_TYDOUBLE 280
#define TK_TYINT 281
#define TK_TYLONG 282
#define TK_TYSHORT 283
#define TK_TYSIGNED 284
#define TK_TYUNSIGNED 285
#define ERRTOK 286
const short ttlhs[] = {                                        -1,
    0,   29,   29,   33,   33,   34,   36,   36,   27,   27,
   38,   35,   30,   30,   30,   39,   40,    2,    2,    2,
    3,    3,    3,    4,    5,    5,    6,   28,   28,   31,
   31,   31,   31,   44,   41,   45,   45,   46,   42,   43,
   43,   37,   37,   37,   18,   18,   18,   19,   19,   19,
   19,   19,   19,   20,   24,   24,   22,   22,   23,   23,
   21,   21,   21,   21,   21,   21,   21,    9,    9,    1,
    1,    1,   32,   32,   32,   11,   14,   14,   12,   13,
   15,   15,   16,   16,   17,   17,   10,    7,    7,    7,
    7,    7,    7,    7,    7,   47,   47,    8,    8,    8,
    8,   25,   25,   26,   26,
};
const short ttlen[] = {                                         2,
    4,    0,    2,    1,    1,    2,    3,    2,    2,    0,
    0,    3,    1,    1,    2,    3,    2,    3,    3,    0,
    3,    3,    3,    3,    1,    0,    4,    1,    0,    2,
    1,    1,    0,    0,    5,    1,    0,    0,    6,    1,
    2,    7,    3,    2,    1,    2,    1,    3,    3,    3,
    2,    1,    0,    2,    2,    0,    1,    3,    1,    3,
    1,    2,    1,    1,    1,    1,    1,    1,    0,    3,
    3,    0,    1,    2,    2,    1,    1,    2,    1,    1,
    1,    2,    1,    2,    1,    2,    1,    1,    1,    2,
    2,    1,    2,    1,    2,    1,    0,    1,    1,    1,
    1,    1,    2,    1,    3,
};
const short ttdefred[] = {                                      2,
    0,    0,    0,   28,    0,    0,   11,    0,    0,    0,
    0,    3,    4,    5,   13,   14,   15,   76,   81,    0,
    0,    0,   87,  104,    0,    6,    0,    0,    0,    0,
   25,   17,    0,   36,    0,    0,    0,   32,    0,   18,
   82,   19,    8,    0,    0,    0,   88,    0,   89,   92,
   94,    0,   99,    0,    0,    0,  102,   47,    0,    0,
   12,   77,    0,    0,    0,    0,   16,    0,    0,   34,
   73,    0,    0,    1,   30,    0,    9,  105,    7,   44,
    0,   93,   90,   91,    0,    0,    0,   96,   98,  100,
  101,  103,   95,   23,   78,   21,   22,   24,   27,    0,
    0,   83,    0,   38,    0,   54,   43,    0,    0,   52,
   40,    0,   84,    0,    0,   51,    0,    0,    0,    0,
   35,   41,    0,   50,   48,   49,   61,   85,   79,   80,
    0,    0,   63,   64,    0,    0,   65,   57,   55,    0,
    0,   39,   59,    0,   62,   86,    0,    0,   58,    0,
    0,   42,   60,    0,    0,
};
const short ttdgoto[] = {                                       1,
  148,    8,   29,   31,   32,    9,   56,   57,  140,   58,
   20,  133,  134,   65,  135,  136,  137,   59,  109,   86,
  138,  139,  144,  120,   60,   25,   45,   10,    2,   11,
   36,   74,   12,   13,   14,   26,  111,   27,   15,   16,
   37,   38,  112,  100,   39,  114,   93,
};
const short ttsindex[] = {                                      0,
    0, -232,   -3,    0, -226, -201,    0, -223, -203, -190,
 -166,    0,    0,    0,    0,    0,    0,    0,    0,   33,
  -51,   37,    0,    0,   55,    0, -218,  -45, -203, -160,
    0,    0, -153,    0,  -16, -168, -161,    0, -159,    0,
    0,    0,    0,  -20,   50,   51,    0, -153,    0,    0,
    0, -164,    0, -167, -158,   60,    0,    0, -153, -210,
    0,    0,   61, -141,   63,  -49,    0,  -46,   65,    0,
    0, -160, -139,    0,    0,    3,    0,    0,    0,    0,
   82,    0,    0,    0, -131,   71,  -37,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, -218,
 -130,    0, -133,    0, -153,    0,    0,  -33,   72,    0,
    0, -118,    0, -218,   41,    0,   42,  -23,  -40, -160,
    0,    0, -107,    0,    0,    0,    0,    0,    0,    0,
  -40, -184,    0,    0, -130, -133,    0,    0,    0, -121,
 -130,    0,    0,  -30,    0,    0,   76,   80,    0,  -40,
 -181,    0,    0, -130,   82,
};
const short ttrindex[] = {                                      0,
    0, -180,    0,    0,    0,    0,    0,    0, -240,    0,
 -182,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,   83,    0,    0,    0, -240,    0,
    0,    0,    0,    0,    0,    0, -177,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  -52,    0,  -47,  -41, -116,    0,    0,    0,  -13,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 -115,    0,    0,    0,    0,    0,  -57,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  145,    0,  146,    0,    0,    0,    0,    0,  -56,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  -10,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  -12,  -43,    0,    0,    0,   88,
   -9,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,   89,   91,
};
const short ttgindex[] = {                                      0,
    0,    0,    0,    0,  124,    0,    0,   94,    0,    9,
  -60,    0,    0,    0,    7,   84,    0,    0,    0,   68,
    0,  -98,    0,    0,    0,  -28,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,   -8,    0,    0,    0,
    0,  119,   54,    0,    0,    0,    0,
};
#define YYTABLESIZE 248
const short tttable[] = {                                      64,
   67,   53,   56,   53,  132,   98,  121,   42,  115,   97,
  100,   21,   98,  150,   24,   67,  101,  142,   61,   81,
   85,   77,  105,    3,  106,   26,   26,   26,   26,   26,
   26,   66,  143,   18,   66,   19,   68,   46,    4,   47,
   23,   69,    5,    6,   97,    7,   66,  117,   69,   68,
   28,  153,   78,  108,   22,   17,   24,   23,   48,  116,
   49,   50,   51,   52,   53,   54,   55,   87,   30,  126,
   88,   89,   53,   90,   91,  145,  146,   23,  101,  118,
   19,   67,  131,   33,   33,   33,   33,   37,   31,   31,
   31,   40,   37,   20,  149,   43,   29,   71,   72,   73,
   44,   19,   34,  122,   35,   23,   70,   34,   79,   80,
   76,   83,   66,   78,  122,   82,   24,   85,   95,   94,
   84,   96,  155,   99,  102,  104,  141,  105,   18,  107,
  113,   41,  119,  124,  125,  147,  151,   46,  152,   47,
   23,   10,   45,   46,   74,   75,   72,   70,   46,   71,
   47,   23,   67,   92,  110,   75,  103,  154,   48,   24,
   49,   50,   51,   52,   53,   54,   55,  123,    0,   48,
    0,   49,   50,   51,   52,   53,   54,   55,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,   53,
   56,    0,    0,    0,   53,   56,   98,    0,    0,    0,
   41,  100,   41,   67,   62,   41,   19,  101,   67,  127,
  128,   19,  129,  102,  130,   23,   18,   63,   98,   98,
   98,   98,   98,  100,  100,  100,  100,  100,   23,  101,
  101,  101,  101,  101,   66,   97,   69,   68,
};
const short ttcheck[] = {                                      45,
   44,   59,   59,   61,   45,   58,  125,   59,   42,   59,
   58,    5,   59,   44,    6,   59,   58,  125,   27,   48,
   58,   42,   46,  256,   85,  266,  267,  268,  269,  270,
  271,   44,  131,  260,   28,  262,   30,  256,  271,  258,
  259,   33,  275,  276,   58,  278,   59,  108,   59,   59,
  274,  150,   44,   91,  256,   59,   48,  259,  277,   93,
  279,  280,  281,  282,  283,  284,  285,   59,  272,   93,
  281,  282,  283,  284,  285,  260,  261,  259,   72,  108,
  262,  125,  123,  266,  267,  268,  277,  270,  266,  267,
  268,   59,  270,  274,  125,   59,  277,  266,  267,  268,
   46,  262,  269,  112,  271,  259,  123,  269,   59,   59,
  270,  279,  125,  105,  123,  280,  108,   58,  260,   59,
  279,   59,  151,   59,  264,  123,  120,   46,  260,   59,
  264,  262,   61,   93,   93,  257,   61,  256,   59,  258,
  259,   59,  259,  259,    0,    0,   59,   59,  256,   59,
  258,  259,   29,   60,   87,   37,   73,  151,  277,  151,
  279,  280,  281,  282,  283,  284,  285,  114,   -1,  277,
   -1,  279,  280,  281,  282,  283,  284,  285,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  257,
  257,   -1,   -1,   -1,  262,  262,  259,   -1,   -1,   -1,
  262,  259,  262,  257,  260,  262,  262,  259,  262,  260,
  261,  262,  263,  264,  265,  259,  260,  273,  281,  282,
  283,  284,  285,  281,  282,  283,  284,  285,  259,  281,
  282,  283,  284,  285,  257,  259,  257,  257,
};
#define YYFINAL 1
#ifndef YYDEBUG
#define YYDEBUG 0
#endif
#define YYMAXTOKEN 286
#if YYDEBUG
const char * const ttname[] = {
"end-of-file",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,"'*'",0,"','","'-'","'.'",0,0,0,0,0,0,0,0,0,0,0,"':'","';'",0,
"'='",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"'['",0,"']'",0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"'{'",0,"'}'",0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,"TK_DELIMITER","TK_TYPENAME","TK_NAME","TK_INTEGER","TK_DOUBLE",
"TK_STRING","TK_CHAR","TK_WSTRING","TK_WCHAR","TK_FORMAT","TK_FORMAT_STRING",
"TK_FORMAT_WSTRING","TK_ALIGNED","TK_ATTRIBUTES","TK_CONST","TK_DESCRIPTION",
"TK_DEFAULT","TK_EVENT_TYPE","TK_FACILITY","TK_IMPORT","TK_STRUCT","TK_TYPEDEF",
"TK_TYCHAR","TK_TYDOUBLE","TK_TYINT","TK_TYLONG","TK_TYSHORT","TK_TYSIGNED",
"TK_TYUNSIGNED","ERRTOK",
};
const char * const ttrule[] = {
"$accept : template",
"template : importSection headerSection attributesSection tmplFormatSpec",
"importSection :",
"importSection : importSection importOrTypedef",
"importOrTypedef : importStmt",
"importOrTypedef : typedefStmt",
"importStmt : TK_IMPORT importPredicate",
"importPredicate : scopedIdentifier optionalStar ';'",
"importPredicate : error ';'",
"optionalStar : '.' '*'",
"optionalStar :",
"$$1 :",
"typedefStmt : TK_TYPEDEF $$1 attributeStmt",
"headerSection : recordHeader",
"headerSection : structHeader",
"headerSection : error ';'",
"recordHeader : facilityStmt eventTypeStmt optionalDescription",
"structHeader : structHeaderStmt optionalDescription",
"facilityStmt : TK_FACILITY integerConstant ';'",
"facilityStmt : TK_FACILITY stringLiteral ';'",
"facilityStmt :",
"eventTypeStmt : TK_EVENT_TYPE signedIntegerConstant ';'",
"eventTypeStmt : TK_EVENT_TYPE stringLiteral ';'",
"eventTypeStmt : TK_EVENT_TYPE TK_DEFAULT ';'",
"description : TK_DESCRIPTION stringLiteral ';'",
"optionalDescription : description",
"optionalDescription :",
"structHeaderStmt : optionalStructFlags TK_STRUCT identifier ';'",
"optionalStructFlags : TK_CONST",
"optionalStructFlags :",
"attributesSection : constAttributes recordAttributes",
"attributesSection : constAttributes",
"attributesSection : recordAttributes",
"attributesSection :",
"$$2 :",
"constAttributes : TK_CONST '{' $$2 attributeStmts '}'",
"optionalAligned : TK_ALIGNED",
"optionalAligned :",
"$$3 :",
"recordAttributes : optionalAligned TK_ATTRIBUTES '{' $$3 attributeStmts '}'",
"attributeStmts : attributeStmt",
"attributeStmts : attributeStmts attributeStmt",
"attributeStmt : typeSpec identifier dimensionSpec initSpec attFormatSpec delimiterSpec ';'",
"attributeStmt : typeName bitField ';'",
"attributeStmt : error ';'",
"typeSpec : typeName",
"typeSpec : TK_STRUCT scopedIdentifier",
"typeSpec : identifier",
"dimensionSpec : '[' integerConstant ']'",
"dimensionSpec : '[' scopedIdentifier ']'",
"dimensionSpec : '[' '*' ']'",
"dimensionSpec : '[' ']'",
"dimensionSpec : bitField",
"dimensionSpec :",
"bitField : ':' integerConstant",
"initSpec : '=' initializer",
"initSpec :",
"initializer : scalarConstant",
"initializer : '{' initializerList '}'",
"initializerList : initializer",
"initializerList : initializerList ',' initializer",
"scalarConstant : TK_INTEGER",
"scalarConstant : '-' TK_INTEGER",
"scalarConstant : charConstant",
"scalarConstant : wcharConstant",
"scalarConstant : doubleConstant",
"scalarConstant : stringLiteral",
"scalarConstant : wstringLiteral",
"attFormatSpec : stringLiteral",
"attFormatSpec :",
"delimiterSpec : TK_DELIMITER '=' stringLiteral",
"delimiterSpec : TK_DELIMITER '=' scopedIdentifier",
"delimiterSpec :",
"tmplFormatSpec : TK_FORMAT",
"tmplFormatSpec : TK_FORMAT_STRING stringLiteral",
"tmplFormatSpec : TK_FORMAT_WSTRING wstringLiteral",
"integerConstant : TK_INTEGER",
"signedIntegerConstant : TK_INTEGER",
"signedIntegerConstant : '-' TK_INTEGER",
"charConstant : TK_CHAR",
"wcharConstant : TK_WCHAR",
"stringLiteral : TK_STRING",
"stringLiteral : stringLiteral TK_STRING",
"wstringLiteral : TK_WSTRING",
"wstringLiteral : wstringLiteral TK_WSTRING",
"doubleConstant : TK_DOUBLE",
"doubleConstant : '-' TK_DOUBLE",
"identifier : TK_NAME",
"typeName : TK_TYPENAME",
"typeName : TK_TYCHAR",
"typeName : TK_TYSIGNED TK_TYCHAR",
"typeName : TK_TYUNSIGNED TK_TYCHAR",
"typeName : TK_TYDOUBLE",
"typeName : TK_TYLONG TK_TYDOUBLE",
"typeName : TK_TYINT",
"typeName : intModifiers optionalInt",
"optionalInt : TK_TYINT",
"optionalInt :",
"intModifier : TK_TYLONG",
"intModifier : TK_TYSHORT",
"intModifier : TK_TYSIGNED",
"intModifier : TK_TYUNSIGNED",
"intModifiers : intModifier",
"intModifiers : intModifiers intModifier",
"scopedIdentifier : identifier",
"scopedIdentifier : scopedIdentifier '.' identifier",
};
#endif
#if YYDEBUG
#include <stdio.h>
#endif
#ifdef YYSTACKSIZE
#undef YYMAXDEPTH
#define YYMAXDEPTH YYSTACKSIZE
#else
#ifdef YYMAXDEPTH
#define YYSTACKSIZE YYMAXDEPTH
#else
#define YYSTACKSIZE 10000
#define YYMAXDEPTH 10000
#endif
#endif
#define YYINITSTACKSIZE 200
int yydebug;
int yynerrs;
int yyerrflag;
int yychar;
short *yyssp;
YYSTYPE *yyvsp;
YYSTYPE yyval;
YYSTYPE yylval;
short *yyss;
short *yysslim;
YYSTYPE *yyvs;
int yystacksize;

void
yyerror(char *s)
{
	fprintf(parserContext.pc_errfile, "%s:%d: %s\n",
		parserContext.pc_pathname, parserContext.pc_lineno, s);
}

void
_evlTmplClearParserErrors()
{
	parserContext.pc_errors = 0;
}

tmpl_parser_context_t *
_evlTmplGetParserContext()
{
	return &parserContext;
}

/*
 * ws is an array of wide characters terminating in L'\0'.  Malloc room
 * for the corresponding multibyte string, write the multibyte string there,
 * and return a pointer to it.  Returns NULL if we can't do the conversion.
 */
static char *
wcsToMbs(wchar_t *ws)
{
	size_t nb1, nb2;
	char *mbs;
	nb1 = wcstombs(NULL, ws, 0);
	if (nb1 == (size_t)-1) {
		return NULL;
	}
	mbs = (char*) malloc(nb1+1);
	assert(mbs != NULL);
	nb2 = wcstombs(mbs, ws, nb1+1);
	assert(nb1 == nb2);
	return mbs;
}

/*
 * modifiers is a list of words from the following set: long, short, signed,
 * unsigned.  Return the implied type code -- e.g., TY_ULONGLONG for
 * {unsigned,long,long} or {long,unsigned,long} or {long,long,unsigned}.
 * On error, call _evlTmplSemanticError() and return TY_INT.
 */
static int
makeIntType(evl_list_t *modifiers)
{
	int size = TY_INT;	/* or TY_LONG, TY_LONGLONG, or TY_SHORT */
	int sign = -1;		/* or TK_TYSIGNED or TK_TYUNSIGNED */
	evl_list_t *head = modifiers, *end, *p;

	for (p=head, end=NULL; p!=end; end=head, p=p->li_next) {
		/* Note: s390x requires double cast here. */
		int modifier = (int) (long) p->li_data;
		switch (modifier) {
		case TK_TYLONG:
			if (size == TY_LONG) {
				size = TY_LONGLONG;
			} else if (size == TY_INT) {
				size = TY_LONG;
			} else {
				goto badType;
			}
			break;
		case TK_TYSHORT:
			if (size == TY_INT) {
				size = TY_SHORT;
			} else {
				goto badType;
			}
			break;
		case TK_TYSIGNED:
		case TK_TYUNSIGNED:
			if (sign == -1) {
				sign = modifier;
			} else {
				goto badType;
			}
			break;
		default:
			assert(0);
		}
	}
	if (sign == TK_TYUNSIGNED) {
		switch (size) {
		case TY_INT:		return TY_UINT;
		case TY_SHORT:		return TY_USHORT;
		case TY_LONG:		return TY_ULONG;
		case TY_LONGLONG:	return TY_ULONGLONG;
		default:		assert(0);
		}
	} else {
		return size;
	}

badType:
	_evlTmplSemanticError("Nonsensical integer type");
	return TY_INT;
}
/* allocate initial stack or double stack size, up to YYMAXDEPTH */
static int yygrowstack()
{
    int newsize, i;
    short *newss;
    YYSTYPE *newvs;

    if ((newsize = yystacksize) == 0)
        newsize = YYINITSTACKSIZE;
    else if (newsize >= YYMAXDEPTH)
        return -1;
    else if ((newsize *= 2) > YYMAXDEPTH)
        newsize = YYMAXDEPTH;
    i = yyssp - yyss;
    newss = yyss ? (short *)realloc(yyss, newsize * sizeof *newss) :
      (short *)malloc(newsize * sizeof *newss);
    if (newss == NULL)
        return -1;
    yyss = newss;
    yyssp = newss + i;
    newvs = yyvs ? (YYSTYPE *)realloc(yyvs, newsize * sizeof *newvs) :
      (YYSTYPE *)malloc(newsize * sizeof *newvs);
    if (newvs == NULL)
        return -1;
    yyvs = newvs;
    yyvsp = newvs + i;
    yystacksize = newsize;
    yysslim = yyss + newsize - 1;
    return 0;
}

#define YYABORT goto yyabort
#define YYREJECT goto yyabort
#define YYACCEPT goto yyaccept
#define YYERROR goto yyerrlab

#ifndef YYPARSE_PARAM
#if defined(__cplusplus) || __STDC__
#define YYPARSE_PARAM_ARG void
#define YYPARSE_PARAM_DECL
#else	/* ! ANSI-C/C++ */
#define YYPARSE_PARAM_ARG
#define YYPARSE_PARAM_DECL
#endif	/* ANSI-C/C++ */
#else	/* YYPARSE_PARAM */
#ifndef YYPARSE_PARAM_TYPE
#define YYPARSE_PARAM_TYPE void *
#endif
#if defined(__cplusplus) || __STDC__
#define YYPARSE_PARAM_ARG YYPARSE_PARAM_TYPE YYPARSE_PARAM
#define YYPARSE_PARAM_DECL
#else	/* ! ANSI-C/C++ */
#define YYPARSE_PARAM_ARG YYPARSE_PARAM
#define YYPARSE_PARAM_DECL YYPARSE_PARAM_TYPE YYPARSE_PARAM;
#endif	/* ANSI-C/C++ */
#endif	/* ! YYPARSE_PARAM */

int
yyparse (YYPARSE_PARAM_ARG)
    YYPARSE_PARAM_DECL
{
    register int yym, yyn, yystate;
#if YYDEBUG
    register const char *yys;

    if ((yys = getenv("YYDEBUG")))
    {
        yyn = *yys;
        if (yyn >= '0' && yyn <= '9')
            yydebug = yyn - '0';
    }
#endif

    yynerrs = 0;
    yyerrflag = 0;
    yychar = (-1);

    if (yyss == NULL && yygrowstack()) goto yyoverflow;
    yyssp = yyss;
    yyvsp = yyvs;
    *yyssp = yystate = 0;

yyloop:
    if ((yyn = yydefred[yystate])) goto yyreduce;
    if (yychar < 0)
    {
        if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, reading %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
    }
    if ((yyn = yysindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: state %d, shifting to state %d\n",
                    YYPREFIX, yystate, yytable[yyn]);
#endif
        if (yyssp >= yysslim && yygrowstack())
        {
            goto yyoverflow;
        }
        *++yyssp = yystate = yytable[yyn];
        *++yyvsp = yylval;
        yychar = (-1);
        if (yyerrflag > 0)  --yyerrflag;
        goto yyloop;
    }
    if ((yyn = yyrindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
        yyn = yytable[yyn];
        goto yyreduce;
    }
    if (yyerrflag) goto yyinrecovery;
#if defined(lint) || defined(__GNUC__)
    goto yynewerror;
#endif
yynewerror:
    yyerror("syntax error");
#if defined(lint) || defined(__GNUC__)
    goto yyerrlab;
#endif
yyerrlab:
    ++yynerrs;
yyinrecovery:
    if (yyerrflag < 3)
    {
        yyerrflag = 3;
        for (;;)
        {
            if ((yyn = yysindex[*yyssp]) && (yyn += YYERRCODE) >= 0 &&
                    yyn <= YYTABLESIZE && yycheck[yyn] == YYERRCODE)
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: state %d, error recovery shifting\
 to state %d\n", YYPREFIX, *yyssp, yytable[yyn]);
#endif
                if (yyssp >= yysslim && yygrowstack())
                {
                    goto yyoverflow;
                }
                *++yyssp = yystate = yytable[yyn];
                *++yyvsp = yylval;
                goto yyloop;
            }
            else
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: error recovery discarding state %d\n",
                            YYPREFIX, *yyssp);
#endif
                if (yyssp <= yyss) goto yyabort;
                --yyssp;
                --yyvsp;
            }
        }
    }
    else
    {
        if (yychar == 0) goto yyabort;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, error recovery discards token %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
        yychar = (-1);
        goto yyloop;
    }
yyreduce:
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: state %d, reducing by rule %d (%s)\n",
                YYPREFIX, yystate, yyn, yyrule[yyn]);
#endif
    yym = yylen[yyn];
    yyval = yyvsp[1-yym];
    switch (yyn)
    {
case 6:
{}
break;
case 7:
{
				(void) _evlImportTemplateFromIdList(yyvsp[-2].listval, yyvsp[-1].cval);
				_evlFreeList(yyvsp[-2].listval, 1);
			}
break;
case 9:
{ yyval.cval = 1;}
break;
case 10:
{ yyval.cval = 0; }
break;
case 11:
{ attrClass=EVL_ATTR_TYPEDEF; }
break;
case 15:
{
				template = _evlMakeEvrecTemplate(0, 0, NULL);
				template->tm_flags |= TMPL_TF_ERROR;
				parserContext.pc_template = template;
			}
break;
case 16:
{
				template = _evlMakeEvrecTemplate(yyvsp[-2].facval, yyvsp[-1].ival, yyvsp[0].sval);
				if (parserContext.pc_errors) {
					template->tm_flags |= TMPL_TF_ERROR;
				}
				parserContext.pc_template = template;
			}
break;
case 17:
{
				template = _evlMakeStructTemplate(yyvsp[-1].sval, yyvsp[0].sval);
				if (parserContext.pc_errors) {
					template->tm_flags |= TMPL_TF_ERROR;
				}
				template->tm_flags |= earlyTmplFlags;
				parserContext.pc_template = template;
			}
break;
case 18:
{
				posix_log_facility_t fac = (posix_log_facility_t) yyvsp[-1].uival;
				
				if (yyvsp[-1].uival > UINT_MAX) {
					_evlTmplSemanticError(
						"Facility code out of range: %lu", yyvsp[-1].uival);
				}
				yyval.facval = fac;
				if (fac == EVL_INVALID_FACILITY) {
					_evlTmplSemanticError(
						"Invalid facility code: %u", fac);
				} else {
					evltc_default_facility = fac;
				}
			}
break;
case 19:
{
				posix_log_facility_t fac;
				if (posix_log_strtofac(yyvsp[-1].sval, &fac) != 0) {
					_evlTmplSemanticError(
						"Unknown facility: %s", yyvsp[-1].sval);
					yyval.facval = EVL_INVALID_FACILITY;
				} else {
					yyval.facval = fac;
					evltc_default_facility = fac;
				}
			}
break;
case 20:
{
				if (evltc_default_facility == 
				    EVL_INVALID_FACILITY) {
					_evlTmplSemanticError(
						"No valid facility specified.");
				}
				yyval.facval = evltc_default_facility;
			}
break;
case 21:
{
				yyval.ival = (int) yyvsp[-1].lval;
				if (yyvsp[-1].lval == EVL_INVALID_EVENT_TYPE) {
					_evlTmplSemanticError(
						"Invalid event type: %d", yyvsp[-1].lval);
				}
			}
break;
case 22:
{
				yyval.ival = evl_gen_event_type_v2(yyvsp[-1].sval);
				if (yyval.ival == EVL_INVALID_EVENT_TYPE) {
					_evlTmplSemanticError(
						"String maps to invalid event type (%d)",
						EVL_INVALID_EVENT_TYPE);
				}
			}
break;
case 23:
{
				yyval.ival = EVL_DEFAULT_EVENT_TYPE;
			}
break;
case 24:
{
				yyval.sval = yyvsp[-1].sval;
			}
break;
case 26:
{ yyval.sval = NULL; }
break;
case 27:
{
				yyval.sval = yyvsp[-1].sval;
				if (_evlFindStructRef(yyvsp[-1].sval)) {
					_evlTmplSemanticError(
"struct %s previously defined or imported", yyvsp[-1].sval);
				}
			}
break;
case 28:
{ earlyTmplFlags = TMPL_TF_CONST; }
break;
case 29:
{ earlyTmplFlags = 0x0; }
break;
case 34:
{ attrClass=EVL_ATTR_CONST; }
break;
case 36:
{ template->tm_flags |= TMPL_TF_ALIGNED; }
break;
case 38:
{ attrClass=0; }
break;
case 42:
{
				_evlTmplAddAttribute(template, attrClass,
					yyvsp[-6].tyval, yyvsp[-5].sval, yyvsp[-4].dimval, yyvsp[-3].valval, yyvsp[-2].sval, yyvsp[-1].delimval);
			}
break;
case 43:
{
				/* Unnamed bit-field */
				tmpl_data_type_t *t = _evlTmplAllocDataType();
				tmpl_dimension_t *dim = _evlTmplAllocDimension();
				t->tt_base_type = yyvsp[-2].ival;
				dim->td_type = TMPL_DIM_BITFIELD;
				dim->td_dimension = yyvsp[-1].ival;
				_evlTmplAddAttribute(template, attrClass, t,
					NULL, dim, NULL, NULL, NULL);
			}
break;
case 45:
{
				tmpl_data_type_t *t = _evlTmplAllocDataType();
				t->tt_base_type = yyvsp[0].ival;
				yyval.tyval = t;
			}
break;
case 46:
{
				extern char *_evlMakeSlashPathFromList(
					evl_list_t *list);
				tmpl_data_type_t *t = _evlTmplAllocDataType();
				t->tt_base_type = TY_STRUCTNAME;
				t->u.st_name = _evlMakeSlashPathFromList(yyvsp[0].listval);
				_evlFreeList(yyvsp[0].listval, 1);
				yyval.tyval = t;
			}
break;
case 47:
{
				tmpl_data_type_t *t = _evlTmplAllocDataType();
				t->tt_base_type = TY_TYPEDEF;
				t->u.td_name = yyvsp[0].sval;
				yyval.tyval = t;
			}
break;
case 48:
{
				tmpl_dimension_t *dim;
				if ( yyvsp[-1].uival > INT_MAX ) {
					_evlTmplSemanticError("Dimension out of range: %lu", yyvsp[-1].uival);
				}
				dim = _evlTmplAllocDimension();
				dim->td_type = TMPL_DIM_CONST;
				dim->td_dimension = (int) yyvsp[-1].uival;
				yyval.dimval = dim;
			}
break;
case 49:
{
				/*
				 * Note that we throw away some info here by
				 * changing { x, y, z } back to "x.y.z".
				 * This shouldn't be a big deal so long as
				 * the use of struct members as dimensions
				 * is fairly rare OR _evlTmplFindAttribute()
				 * is quick.  The latter, at least, seems to
				 * be true.  If we go to a more efficient
				 * implementation here, also consider using
				 * it for attribute refs in the template's
				 * format section.
				 */
				tmpl_dimension_t *dim = _evlTmplAllocDimension();
				dim->td_type = TMPL_DIM_ATTNAME;
				dim->u.u_attname = _evlMakeDotPathFromList(yyvsp[-1].listval);
				_evlFreeList(yyvsp[-1].listval, 1);
				yyval.dimval = dim;
			}
break;
case 50:
{
				tmpl_dimension_t *dim = _evlTmplAllocDimension();
				dim->td_type = TMPL_DIM_REST;
				yyval.dimval = dim;
			}
break;
case 51:
{
				tmpl_dimension_t *dim = _evlTmplAllocDimension();
				dim->td_type = TMPL_DIM_IMPLIED;
				dim->td_dimension = 0;
				yyval.dimval = dim;
			}
break;
case 52:
{
				tmpl_dimension_t *dim = _evlTmplAllocDimension();
				dim->td_type = TMPL_DIM_BITFIELD;
				dim->td_dimension = yyvsp[0].ival;
				yyval.dimval = dim;
			}
break;
case 53:
{ yyval.dimval = NULL; }
break;
case 54:
{
				if (yyvsp[0].uival > INT_MAX) {
					_evlTmplSemanticError(
					"bit field size out of range: %lu", yyvsp[0].uival);
				} 
				yyval.ival = (int) yyvsp[0].uival; 
			}
break;
case 55:
{ yyval.valval = yyvsp[0].valval; }
break;
case 56:
{ yyval.valval = NULL; }
break;
case 58:
{ yyval.valval = yyvsp[-1].valval; }
break;
case 59:
{
				/* Initializer -> initializer list */
				tmpl_type_and_value_t *tv = _evlTmplAllocTypeAndValue();
				evl_list_t *list = _evlMkListNode(yyvsp[0].valval);
				tv->tv_type = TY_LIST;
				tv->tv_value.val_list = list;
				yyval.valval = tv;
			}
break;
case 60:
{
				evl_list_t *list = yyvsp[-2].valval->tv_value.val_list;
				(void) _evlAppendToList(list, yyvsp[0].valval);
				yyval.valval = yyvsp[-2].valval;
			}
break;
case 61:
{
				tmpl_type_and_value_t *tv = _evlTmplAllocTypeAndValue();
				tv->tv_type = TY_ULONG;
				tv->tv_value.val_ulong = yyvsp[0].ulval;
				yyval.valval = tv;
			}
break;
case 62:
{
				tmpl_type_and_value_t *tv = _evlTmplAllocTypeAndValue();
				tv->tv_type = TY_LONG;
				tv->tv_value.val_long = (long) -yyvsp[0].ulval;
				yyval.valval = tv;
			}
break;
case 63:
{
				tmpl_type_and_value_t *tv = _evlTmplAllocTypeAndValue();
				tv->tv_type = TY_LONG;
				tv->tv_value.val_long = yyvsp[0].cval;
				yyval.valval = tv;
			}
break;
case 64:
{
				tmpl_type_and_value_t *tv = _evlTmplAllocTypeAndValue();
				tv->tv_type = TY_LONG;
				tv->tv_value.val_long = yyvsp[0].wcval;
				yyval.valval = tv;
			}
break;
case 65:
{
				tmpl_type_and_value_t *tv = _evlTmplAllocTypeAndValue();
				tv->tv_type = TY_DOUBLE;
				tv->tv_value.val_double = yyvsp[0].dval;
				yyval.valval = tv;
			}
break;
case 66:
{
				tmpl_type_and_value_t *tv = _evlTmplAllocTypeAndValue();
				tv->tv_type = TY_STRING;
				tv->tv_value.val_string = yyvsp[0].sval;
				yyval.valval = tv;
			}
break;
case 67:
{
				tmpl_type_and_value_t *tv = _evlTmplAllocTypeAndValue();
				tv->tv_type = TY_WSTRING;
				tv->tv_value.val_wstring = yyvsp[0].wsval;
				yyval.valval = tv;
			}
break;
case 69:
{ yyval.sval = NULL; }
break;
case 70:
{
				tmpl_delimiter_t *de = _evlTmplAllocDelimiter();
				de->de_type = TMPL_DELIM_CONST;
				de->de_delimiter = yyvsp[0].sval;
				yyval.delimval = de;
			}
break;
case 71:
{
				/* Handle like an array dimension. */
				tmpl_delimiter_t *de = _evlTmplAllocDelimiter();
				de->de_type = TMPL_DELIM_ATTNAME; /* for now */
				de->de_delimiter = NULL;
				de->u.u_attname = _evlMakeDotPathFromList(yyvsp[0].listval);
				_evlFreeList(yyvsp[0].listval, 1);
				yyval.delimval = de;
			}
break;
case 72:
{ yyval.delimval = NULL; }
break;
case 73:
{
				_evlTmplAddFormatSpec(template, yyvsp[0].sval);
				_evlTmplWrapup(template);
			}
break;
case 74:
{
				_evlTmplAddFormatSpec(template, yyvsp[0].sval);
				_evlTmplWrapup(template);
			}
break;
case 75:
{
				char *s = wcsToMbs(yyvsp[0].wsval);
				if (s) {
					_evlTmplAddFormatSpec(template, s);
				} else {
					fprintf(stderr,
"Internal error: Cannot convert wstring format specification.\n");
					template->tm_flags |= TMPL_TF_ERROR;
				}
				_evlTmplWrapup(template);
			}
break;
case 76:
{ yyval.uival = yyvsp[0].ulval; }
break;
case 77:
{ yyval.lval = yyvsp[0].ulval; }
break;
case 78:
{ yyval.lval = -yyvsp[0].ulval; }
break;
case 82:
{
				/* Concatenate adjacent string literals. */
				char *s1 = yyvsp[-1].sval;
				char *s2 = yyvsp[0].sval;
				char *snew = realloc(s1,
					strlen(s1)+strlen(s2)+1);
				assert(snew != NULL);
				(void) strcat(snew, s2);
				free(s2);
				yyval.sval = snew;
			}
break;
case 84:
{
				/* Concatenate adjacent wide string literals. */
				wchar_t *s1 = yyvsp[-1].wsval;
				wchar_t *s2 = yyvsp[0].wsval;
				size_t len1 = wcslen(s1);
				size_t len2 = wcslen(s2);
				wchar_t *snew = realloc(s1,
					(len1+len2+1) * sizeof(wchar_t));
				assert(snew != NULL);
				(void) memcpy(snew + len1, s2,
					(len2+1)*sizeof(wchar_t));
				free(s2);
				yyval.wsval = snew;
			}
break;
case 86:
{ yyval.dval = -yyvsp[0].dval; }
break;
case 89:
{
#if (CHAR_MIN == 0)
				yyval.ival = TY_UCHAR;
#else
				yyval.ival = TY_CHAR;
#endif
			}
break;
case 90:
{ yyval.ival = TY_CHAR; }
break;
case 91:
{ yyval.ival = TY_UCHAR; }
break;
case 92:
{ yyval.ival = TY_DOUBLE; }
break;
case 93:
{ yyval.ival = TY_LDOUBLE; }
break;
case 94:
{ yyval.ival = TY_INT; }
break;
case 95:
{
				yyval.ival = makeIntType(yyvsp[-1].listval);
				_evlFreeList(yyvsp[-1].listval, 0);
			}
break;
case 96:
{}
break;
case 97:
{}
break;
case 98:
{ yyval.ival = TK_TYLONG; }
break;
case 99:
{ yyval.ival = TK_TYSHORT; }
break;
case 100:
{ yyval.ival = TK_TYSIGNED; }
break;
case 101:
{ yyval.ival = TK_TYUNSIGNED; }
break;
case 102:
{
				yyval.listval = _evlMkListNode((void*)(yyvsp[0].ival));
			}
break;
case 103:
{
				yyval.listval = _evlAppendToList(yyvsp[-1].listval, (void*)(yyvsp[0].ival));
			}
break;
case 104:
{
				yyval.listval = _evlMkListNode(yyvsp[0].sval);
			}
break;
case 105:
{
				yyval.listval = _evlAppendToList(yyvsp[-2].listval, yyvsp[0].sval);
			}
break;
    }
    yyssp -= yym;
    yystate = *yyssp;
    yyvsp -= yym;
    yym = yylhs[yyn];
    if (yystate == 0 && yym == 0)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: after reduction, shifting from state 0 to\
 state %d\n", YYPREFIX, YYFINAL);
#endif
        yystate = YYFINAL;
        *++yyssp = YYFINAL;
        *++yyvsp = yyval;
        if (yychar < 0)
        {
            if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
            if (yydebug)
            {
                yys = 0;
                if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
                if (!yys) yys = "illegal-symbol";
                printf("%sdebug: state %d, reading %d (%s)\n",
                        YYPREFIX, YYFINAL, yychar, yys);
            }
#endif
        }
        if (yychar == 0) goto yyaccept;
        goto yyloop;
    }
    if ((yyn = yygindex[yym]) && (yyn += yystate) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yystate)
        yystate = yytable[yyn];
    else
        yystate = yydgoto[yym];
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: after reduction, shifting from state %d \
to state %d\n", YYPREFIX, *yyssp, yystate);
#endif
    if (yyssp >= yysslim && yygrowstack())
    {
        goto yyoverflow;
    }
    *++yyssp = yystate;
    *++yyvsp = yyval;
    goto yyloop;
yyoverflow:
    yyerror("yacc stack overflow");
yyabort:
    return (1);
yyaccept:
    return (0);
}
